@@ -22,10 +22,8 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-# HUBzero is a registered trademark of HUBzero Foundation, LLC.
-#
-
-from optparse import OptionParser,SUPPRESS_HELP
+from optparse import OptionParser,SUPPRESS_HELP,IndentedHelpFormatter
+import textwrap
 #import sys
 import os.path
 import re
@@ -38,6 +36,79 @@ from hubzero.submit.LogMessage    import logID as log
 from hubzero.submit.CommentedFile import CommentedFile
 
 class CommandParser:
+
+   class IndentedHelpFormatterWithNL(IndentedHelpFormatter):
+      def _format_text(self, text):
+         """
+         Format a paragraph of free-form text for inclusion in the
+         help output at the current indentation level.
+         """
+         result = []
+         text_width = self.width - self.current_indent
+
+         text_text = '\n'.join([x.strip('\n\x0b\x0c\r ') for x in text.split('\n')])
+         for para in text_text.split('\n\n\n'):
+            if '\n\n' in para:
+               for block in para.split('\n\n'):
+                  text_lines = textwrap.wrap(block.lstrip('\n'), text_width)
+                  if len(text_lines):
+                     for line in text_lines:
+                        result.append("%*s%s\n" % (self.current_indent, "", line))
+               result.append('\n')
+            else:
+               text_lines = textwrap.wrap(para.lstrip('\n'), text_width)
+               if len(text_lines):
+                  for line in text_lines:
+                     result.append("%*s%s\n" % (self.current_indent, "", line))
+                  result.append('\n')
+
+         return "".join(result)
+
+
+      def format_option(self, option):
+         # The help for each option consists of two parts:
+         #   * the opt strings and metavars
+         #   eg. ("-x", or "-fFILENAME, --file=FILENAME")
+         #   * the user-supplied help string
+         #   eg. ("turn on expert mode", "read data from FILENAME")
+         #
+         # If possible, we write both of these on the same line:
+         #   -x    turn on expert mode
+         #
+         # But if the opt string list is too long, we put the help
+         # string on a second line, indented to the same column it would
+         # start in if it fit on the first line.
+         #   -fFILENAME, --file=FILENAME
+         #           read data from FILENAME
+         result = []
+         opts = self.option_strings[option]
+         opt_width = self.help_position - self.current_indent - 2
+         if len(opts) > opt_width:
+            opts = "%*s%s\n" % (self.current_indent, "", opts)
+            indent_first = self.help_position
+         else:                       # start help on same line as opts
+            opts = "%*s%-*s  " % (self.current_indent, "", opt_width, opts)
+            indent_first = 0
+         result.append(opts)
+         if option.help:
+            help_text = self.expand_default(option)
+            if '\n\n' in help_text:
+               help_lines = []
+               help_text = "\n".join([x.strip() for x in help_text.split("\n")])
+               for para in help_text.split("\n\n"):
+                  help_lines.extend(textwrap.wrap(para, self.help_width))
+               if len(help_lines):
+                  help_lines[-1] += "\n"
+            else:
+               help_lines = textwrap.wrap(help_text, self.help_width)
+            result.append("%*s%s\n" % (indent_first, "", help_lines[0]))
+            result.extend(["%*s%s\n" % (self.help_position, "", line) for line in help_lines[1:]])
+         elif opts[-1] != "\n":
+            result.append("\n")
+
+         return "".join(result)
+
+
    def __grabMultipleArguments(self,option,opt_str,value,parser):
       assert value is None
       value = getattr(parser.values,option.dest)
@@ -215,21 +286,115 @@ class CommandParser:
       self.csvDataHasHeader     = False
       self.nCSVDataCombinations = 0
 
-      self.parser = OptionParser(prog="submit",add_help_option=False)
+      exampleDescriptions = """Parameter examples:
+\n\n
+submit -p @@cap=10pf,100pf,1uf sim.exe @:indeck
+\n\n
+\tSubmit 3 jobs. The @:indeck means "use the file indeck as a template file."
+Substitute the values 10pf, 100pf, and 1uf in place of @@cap within the file.
+Send off one job for each of the values and bring back the results.
+\n\n
+submit -p @@vth=0:0.2:5 -p @@cap=10pf,100pf,1uf sim.exe @:indeck
+\n\n
+\tSubmit 78 jobs. The parameter @@vth goes from 0 to 5 in steps of 0.2, so there are 26 values for @@vth.
+For each of those values, the parameter @@cap changes from 10pf to 100pf to 1uf. 26 x 3 = 78 jobs total.
+Again @:indeck is treated as a template, and the values are substituted in place of @@vth and @@cap in that file.
+\n\n
+submit -p params sim.exe @:indeck
+\n\n
+\tIn this case, parameter definitions are taken from the file named params instead of the command line.
+The file might have the following contents:
+\n\n
+\t\t# paramters for my job submission\n
+\t\tparameter @@vth=0:0.2:5\n
+\t\tparameter @@cap = 10pf,100pf,1uf\n\t
+\n\n
+submit -p "params;@@num=1-10;@@color=blue" job.sh @:job.data
+\n\n
+\tFor someone who loves syntax and complexity... The semicolon separates the parameters value into three parts.
+The first says to load parameters from a file params.
+The next part says add an additional parameter @@num that goes from 1 to 10.
+The last part says add an additional parameter @@color with a single value blue.
+The parameters @@num and @@color cannot override anything defined within params; they must be new parameter names.
+\n\n
+submit -d input.csv sim.exe @:indeck
+\n\n
+\tTakes parameters from the data file input.csv, which must be in comma-separated value format.
+The first line of this file may contain a series of @@param names for each of the columns.
+If it doesn't, then the columns are assumed to be called @@1, @@2, @@3, etc.
+Each of the remaining lines represents a set of parameter values for one job; if there are 100 such lines, there will be 100 jobs.
+For example, the file input.csv might look like this:
+\n\n
+\t\t@@vth, @@cap\n
+\t\t1.1, 1pf\n
+\t\t2.2, 1pf\n
+\t\t1.1, 10pf\n
+\t\t2.2, 10pf
+\n\n
+\tParameters are substituted as before into template files such as @:indeck.
+\n\n
+submit -d input.csv -p "@@doping=1e15-1e17 in 30 log" sim.exe @:infile
+\n\n
+\tTakes parameters from the data file input.csv, but
+also adds another parameter @@doping which goes from 1e15 to 1e17 in 30 points on a log scale.
+For each of these points, all values in the data file will be executed.
+If the data file specifies 50 jobs, then this command would run 30 x 50 = 1500 jobs.
+\n\n
+submit -d input.csv -i @:extra/data.txt sim.exe @:indeck
+\n\n
+\tIn addition to the template indeck file, send along another file extra/data.txt with each job, and treat it as a template too.
+\n\n
+submit -s / -p @@address=23 Main St.,Hometown,Indiana/42 Broadway,Hometown,Indiana -s , -p @@color=red,green,blue job.sh @:job.data
+\n\n
+\tChange the separator to slash when defining the addresses, then
+change back to comma for the @@color parameter and any remaining arguments.
+We shouldn't have to change the separator often, but it might come in handy if the value strings themselves have commas.
+\n\n
+submit @@num=1:1000 sim.exe input@@num
+\n\n
+\tSubmit jobs 1,2,3,...,1000.
+Parameter names such as @@num are recognized not only in template files, but also for arguments on the command line.
+In this case, the numbers 1,2,3,...,1000 are substituted into the file name,
+so the various jobs take their input from "input1", "input2", ..., "input1000".
+\n\n
+submit @@file=glob:indeck* sim.exe @:file
+\n\n
+\tLook for files matching indeck* and use the list of names as the parameter @@file.
+Those values could be substituted into other template files, or used on the command line as in this example.
+Suppose the directory contains files indeckA, indeckB, and indeck-123.
+This example would launch three jobs using each of those files as input for the job.
+\n\n
+"""
+      indentedHelpFormatterWithNL = self.IndentedHelpFormatterWithNL()
+      self.parser = OptionParser(prog="submit",add_help_option=False,
+                                 epilog=exampleDescriptions,formatter=indentedHelpFormatterWithNL)
       self.parser.disable_interspersed_args()
       self.parser.add_option("-j","--jobid",action="store",type="long",dest="jobId", \
                                             help=SUPPRESS_HELP)
 
-      self.parser.set_defaults(localExecution=False)
-      self.parser.add_option("-l","--local",action="store_true",dest="localExecution", \
-                                            help="Execute command locally")
-
       self.parser.set_defaults(help=False)
       self.parser.set_defaults(helpManagers=False)
       self.parser.set_defaults(helpVenues=False)
       self.parser.set_defaults(helpTools=False)
       allowedHelpOptions = ['managers','venues','tools']
 
+      if doubleDashTerminator:
+         self.parser.add_option("-h","--help",action="callback", \
+                                              callback=self.__grabMultipleHelpArguments,dest="help", \
+                                              callback_kwargs={'allowedHelpOptions':allowedHelpOptions}, \
+                                              help="Report command usage." + \
+                                                   " Optionally request listing of managers, tools, or venues.")
+      else:
+         self.parser.add_option("-h","--help",action="callback", \
+                                              callback=self.__grabHelpArgument,dest="help", \
+                                              callback_kwargs={'allowedHelpOptions':allowedHelpOptions}, \
+                                              help="Report command usage." + \
+                                                   " Optionally request listing of managers, tools, or venues.")
+
+      self.parser.set_defaults(localExecution=False)
+      self.parser.add_option("-l","--local",action="store_true",dest="localExecution", \
+                                            help="Execute command locally")
+
       if doubleDashTerminator:
          self.parser.add_option("-v","--venue",action="callback", \
                                                callback=self.__grabMultipleArguments,dest="destinations", \
@@ -242,11 +407,7 @@ class CommandParser:
                                                     help=SUPPRESS_HELP)
          self.parser.add_option("-p","--parameters",action="callback", \
                                                     callback=self.__grabMultipleParameterArguments,dest="parameters", \
-                                                    help="Parameter sweep variables terminated by --")
-         self.parser.add_option("-h","--help",action="callback", \
-                                              callback=self.__grabMultipleHelpArguments,dest="help", \
-                                              callback_kwargs={'allowedHelpOptions':allowedHelpOptions}, \
-                                              help="Report command usage")
+                                                    help="Parameter sweep variables terminated by --. See examples.")
       else:
          self.parser.add_option("-v","--venue",action="callback", \
                                                callback=self.__grabStringArgument,dest="destinations", \
@@ -259,11 +420,7 @@ class CommandParser:
                                                     help=SUPPRESS_HELP)
          self.parser.add_option("-p","--parameters",action="callback", \
                                                     callback=self.__grabParameterArgument,dest="parameters", \
-                                                    help="Parameter sweep variables")
-         self.parser.add_option("-h","--help",action="callback", \
-                                              callback=self.__grabHelpArgument,dest="help", \
-                                              callback_kwargs={'allowedHelpOptions':allowedHelpOptions}, \
-                                              help="Report command usage")
+                                                    help="Parameter sweep variables. See examples.")
 
       self.parser.add_option("-d","--data",action="callback", \
                                            callback=self.__grabStringArgument,dest="csvData", \
